@@ -874,7 +874,7 @@ DEFINE_SET_FIELD(jint, Int)
 DEFINE_SET_FIELD(jlong, Long)
 DEFINE_SET_FIELD(jshort, Short)
 
-void JCCEnv::setClassPath(const char *classPath)
+bool JCCEnv::setClassPath(const char *classPath)
 {
     JNIEnv *vm_env = get_vm_env();
     jclass _ucl = (jclass) vm_env->FindClass("java/net/URLClassLoader");
@@ -882,6 +882,10 @@ void JCCEnv::setClassPath(const char *classPath)
     jmethodID mid = vm_env->GetStaticMethodID(_ucl, "getSystemClassLoader",
                                               "()Ljava/lang/ClassLoader;");
     jobject classLoader = vm_env->CallStaticObjectMethod(_ucl, mid);
+
+    if (!vm_env->IsInstanceOf(classLoader, _ucl))
+        return false;
+    
     jmethodID mf = vm_env->GetMethodID(_fil, "<init>", "(Ljava/lang/String;)V");
     jmethodID mu = vm_env->GetMethodID(_fil, "toURL", "()Ljava/net/URL;");
     jmethodID ma = vm_env->GetMethodID(_ucl, "addURL", "(Ljava/net/URL;)V");
@@ -903,6 +907,8 @@ void JCCEnv::setClassPath(const char *classPath)
         vm_env->CallVoidMethod(classLoader, ma, url);
     }
     free(path);
+
+    return true;
 }
 
 char *JCCEnv::getClassPath()
@@ -955,6 +961,7 @@ jstring JCCEnv::fromUTF32(const uint32_t *chars, jsize len) const
 {
     std::vector<jchar> jchars;
 
+    jchars.reserve((size_t) len);
     for (jsize i = 0; i < len; ++i) {
         uint32_t c = chars[i];
 
@@ -987,27 +994,18 @@ jstring JCCEnv::fromPyString(PyObject *object) const
         PyUnicode_READY(object);
 
         switch (PyUnicode_KIND(object)) {
-          case PyUnicode_WCHAR_KIND: {  // this code path should be deprecated
-              if (sizeof(Py_UNICODE) == sizeof(jchar))
-              {
-                  Py_ssize_t len;
-                  Py_UNICODE *pchars = PyUnicode_AsUnicodeAndSize(object, &len);
+          case PyUnicode_1BYTE_KIND: {
+              Py_ssize_t len = PyUnicode_GET_LENGTH(object);
+              Py_UCS1 *pchars = PyUnicode_1BYTE_DATA(object);
+              std::vector<jchar> jchars((size_t) len);
 
-                  return get_vm_env()->NewString((jchar *) pchars, (jsize) len);
-              }
-              else  // Py_UNICODE 32-bit case
-              {
-                  Py_ssize_t len;
-                  Py_UNICODE *pchars = PyUnicode_AsUnicodeAndSize(object, &len);
+              for (std::vector<jchar>::iterator it = jchars.begin();
+                   it != jchars.end(); ++it)
+                  *it = (jchar) *(pchars++);
 
-                  return fromUTF32((uint32_t *) pchars, (jsize) len);
-              }
+              return get_vm_env()->NewString(jchars.data(), (size_t) len);
           }
 
-          case PyUnicode_1BYTE_KIND:
-              return get_vm_env()->NewStringUTF(
-                  (const char *) PyUnicode_1BYTE_DATA(object));
-
           case PyUnicode_2BYTE_KIND: {
               Py_ssize_t len = PyUnicode_GET_LENGTH(object);
               Py_UCS2 *pchars = PyUnicode_2BYTE_DATA(object);
@@ -1035,16 +1033,8 @@ jstring JCCEnv::fromPyString(PyObject *object) const
     return NULL;
 }
 
-PyObject *JCCEnv::fromJString(jstring js, int delete_local_ref) const
+PyObject *JCCEnv::fromJChars(const jchar *utf16, jsize len16) const
 {
-    if (!js)
-        Py_RETURN_NONE;
-
-    JNIEnv *vm_env = get_vm_env();
-    jsize len16 = vm_env->GetStringLength(js);
-    jboolean isCopy;
-    const jchar *utf16 = vm_env->GetStringChars(js, &isCopy);
-
     int32_t len32 = 0;
     uint32_t max_char = 0;
 
@@ -1058,10 +1048,8 @@ PyObject *JCCEnv::fromJString(jstring js, int delete_local_ref) const
 
     PyObject *result = PyUnicode_New(len32, max_char);
 
-    if (result == NULL) {
-        vm_env->ReleaseStringChars(js, utf16);
+    if (result == NULL)
         return NULL;
-    }
 
     switch (PyUnicode_KIND(result)) {
       case PyUnicode_1BYTE_KIND:
@@ -1084,13 +1072,22 @@ PyObject *JCCEnv::fromJString(jstring js, int delete_local_ref) const
               PyUnicode_4BYTE_DATA(result)[len32++] = cp;
           }
           break;
-
-      default:
-          Py_DECREF(result);
-          vm_env->ReleaseStringChars(js, utf16);
-          return NULL;
     }
 
+    return result;
+}
+
+PyObject *JCCEnv::fromJString(jstring js, int delete_local_ref) const
+{
+    if (!js)
+        Py_RETURN_NONE;
+
+    JNIEnv *vm_env = get_vm_env();
+    jsize len16 = vm_env->GetStringLength(js);
+    jboolean isCopy;
+    const jchar *utf16 = vm_env->GetStringChars(js, &isCopy);
+    PyObject *result = fromJChars(utf16, len16);
+
     vm_env->ReleaseStringChars(js, utf16);
     if (delete_local_ref)
         vm_env->DeleteLocalRef((jobject) js);
