@@ -12,7 +12,7 @@
 
 import os, sys, platform, subprocess
 
-jcc_ver = '3.5'
+jcc_ver = '3.10'
 machine = platform.machine()
 using_python2 = sys.version_info < (3,)
 
@@ -99,6 +99,8 @@ running setup.py.
 
 INCLUDES = {
     'darwin/frameworks': ['%(darwin)s/Headers' %(JDK)],
+    'darwin/adoptopenjdk': ['%(darwin)s/include' %(JDK),
+                            '%(darwin)s/include/darwin' %(JDK)],
     'darwin/home': ['%(darwin)s/include' %(JDK),
                     '%(darwin)s/include/darwin' %(JDK)],
     'ipod': ['%(ipod)s/darwin/default' %(JDK)],
@@ -139,6 +141,11 @@ DEBUG_CFLAGS = {
 
 LFLAGS = {
     'darwin/frameworks': ['-framework', 'JavaVM', '-mmacosx-version-min=10.9'],
+    'darwin/adoptopenjdk': ['-L%(darwin)s/lib' %(JDK), '-ljava',
+                            '-L%(darwin)s/lib/server' %(JDK), '-ljvm',
+                            '-Wl,-rpath', '-Wl,%(darwin)s/lib' %(JDK),
+                            '-Wl,-rpath', '-Wl,%(darwin)s/lib/server' %(JDK),
+                            '-mmacosx-version-min=10.9'],
     'darwin/home': ['-L%(darwin)s/jre/lib' %(JDK), '-ljava',
                     '-L%(darwin)s/jre/lib/server' %(JDK), '-ljvm',
                     '-Wl,-rpath', '-Wl,%(darwin)s/jre/lib' %(JDK),
@@ -174,14 +181,18 @@ if platform == 'linux':
     LFLAGS['linux'] = LFLAGS['linux/%s' %(machine)]
 elif platform == 'darwin':
     if JAVAHOME is not None:
-        INCLUDES['darwin'] = INCLUDES['darwin/home']
-        LFLAGS['darwin'] = LFLAGS['darwin/home']
+        if 'adoptopenjdk' in JAVAHOME:
+            INCLUDES['darwin'] = INCLUDES['darwin/adoptopenjdk']
+            LFLAGS['darwin'] = LFLAGS['darwin/adoptopenjdk']
+        else:
+            INCLUDES['darwin'] = INCLUDES['darwin/home']
+            LFLAGS['darwin'] = LFLAGS['darwin/home']
     elif JAVAFRAMEWORKS is not None:
         INCLUDES['darwin'] = INCLUDES['darwin/frameworks']
         LFLAGS['darwin'] = LFLAGS['darwin/frameworks']
 
 JAVAC = {
-    'darwin': ['javac', '-source', '1.5', '-target', '1.5'],
+    'darwin': ['javac', '-source', '1.8', '-target', '1.8'],
     'ipod': ['jikes', '-cp', '/usr/share/classpath/glibj.zip'],
     'linux': ['javac'],
     'sunos5': ['javac'],
@@ -364,8 +375,9 @@ def main(debug):
                           '-current_version', jcc_ver,
                           '-compatibility_version', jcc_ver]
         elif platform == 'linux':
-            kwds["extra_link_args"] = \
-                lflags + ['-lpython%s.%s%s' %(
+            kwds["extra_link_args"] = lflags
+            if sys.version_info < (3, 8):
+                kwds["extra_link_args"] += ['-lpython%s.%s%s' %(
                     sys.version_info[0:2] + ('' if using_python2 else 'm',))]
             kwds["force_shared"] = True    # requires jcc/patches/patch.43
         elif platform in IMPLIB_LFLAGS:
