@@ -16,10 +16,15 @@
  */
 package org.apache.lucene.index;
 
+import static com.carrotsearch.randomizedtesting.RandomizedTest.randomBoolean;
+import static com.carrotsearch.randomizedtesting.RandomizedTest.randomLongBetween;
+import static java.util.stream.Collectors.toList;
+
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.Comparator;
 import java.util.List;
 import java.util.Random;
 import java.util.concurrent.atomic.AtomicBoolean;
@@ -28,11 +33,11 @@ import java.util.concurrent.atomic.AtomicInteger;
 import org.apache.lucene.analysis.MockAnalyzer;
 import org.apache.lucene.document.Document;
 import org.apache.lucene.document.Field;
+import org.apache.lucene.document.LongPoint;
 import org.apache.lucene.search.DocIdSetIterator;
 import org.apache.lucene.search.IndexSearcher;
 import org.apache.lucene.search.Query;
 import org.apache.lucene.search.TermQuery;
-import org.apache.lucene.search.TopDocs;
 import org.apache.lucene.store.AlreadyClosedException;
 import org.apache.lucene.store.Directory;
 import org.apache.lucene.store.MockDirectoryWrapper.FakeIOException;
@@ -50,7 +55,7 @@ import org.junit.Test;
 @SuppressCodecs("SimpleText") // too slow here
 public class TestIndexWriterReader extends LuceneTestCase {
   
-  private final int numThreads = TEST_NIGHTLY ? 5 : 3;
+  private final int numThreads = TEST_NIGHTLY ? 5 : 2;
   
   public static int count(Term t, IndexReader r) throws IOException {
     int count = 0;
@@ -60,7 +65,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
         0);
 
     if (td != null) {
-      final Bits liveDocs = MultiFields.getLiveDocs(r);
+      final Bits liveDocs = MultiBits.getLiveDocs(r);
       while (td.nextDoc() != DocIdSetIterator.NO_MORE_DOCS) {
         td.docID();
         if (liveDocs == null || liveDocs.get(td.docID())) {
@@ -242,7 +247,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
     boolean doFullMerge = false;
 
     Directory dir1 = getAssertNoDeletesDirectory(newDirectory());
-    IndexWriterConfig iwc = newIndexWriterConfig(new MockAnalyzer(random()));
+    IndexWriterConfig iwc = newIndexWriterConfig(new MockAnalyzer(random())).setMaxFullFlushMergeWaitMillis(0);
     if (iwc.getMaxBufferedDocs() < 20) {
       iwc.setMaxBufferedDocs(20);
     }
@@ -295,11 +300,11 @@ public class TestIndexWriterReader extends LuceneTestCase {
     boolean doFullMerge = false;
 
     Directory dir1 = getAssertNoDeletesDirectory(newDirectory());
-    IndexWriter writer = new IndexWriter(dir1, newIndexWriterConfig(new MockAnalyzer(random())));
+    IndexWriter writer = new IndexWriter(dir1, newIndexWriterConfig(new MockAnalyzer(random())).setMaxFullFlushMergeWaitMillis(0));
 
     // create a 2nd index
     Directory dir2 = newDirectory();
-    IndexWriter writer2 = new IndexWriter(dir2, newIndexWriterConfig(new MockAnalyzer(random())));
+    IndexWriter writer2 = new IndexWriter(dir2, newIndexWriterConfig(new MockAnalyzer(random())).setMaxFullFlushMergeWaitMillis(0));
     createIndexNoClose(!doFullMerge, "index2", writer2);
     writer2.close();
 
@@ -325,7 +330,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
     boolean doFullMerge = true;
 
     Directory dir1 = getAssertNoDeletesDirectory(newDirectory());
-    IndexWriter writer = new IndexWriter(dir1, newIndexWriterConfig(new MockAnalyzer(random())));
+    IndexWriter writer = new IndexWriter(dir1, newIndexWriterConfig(new MockAnalyzer(random())).setMaxFullFlushMergeWaitMillis(0));
     // create the index
     createIndexNoClose(!doFullMerge, "index1", writer);
     writer.flush(false, true);
@@ -362,7 +367,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
     writer.close();
         
     // reopen the writer to verify the delete made it to the directory
-    writer = new IndexWriter(dir1, newIndexWriterConfig(new MockAnalyzer(random())));
+    writer = new IndexWriter(dir1, newIndexWriterConfig(new MockAnalyzer(random())).setMaxFullFlushMergeWaitMillis(0));
     IndexReader w2r1 = writer.getReader();
     assertEquals(0, count(new Term("id", id10), w2r1));
     w2r1.close();
@@ -372,13 +377,14 @@ public class TestIndexWriterReader extends LuceneTestCase {
 
   @Slow
   public void testAddIndexesAndDoDeletesThreads() throws Throwable {
-    final int numIter = 2;
-    int numDirs = 3;
+    final int numIter = TEST_NIGHTLY ? 2 : 1;
+    int numDirs = TEST_NIGHTLY ? 3 : 2;
     
     Directory mainDir = getAssertNoDeletesDirectory(newDirectory());
 
     IndexWriter mainWriter = new IndexWriter(mainDir, newIndexWriterConfig(new MockAnalyzer(random()))
-                                                        .setMergePolicy(newLogMergePolicy()));
+                                                        .setMergePolicy(newLogMergePolicy())
+                                                        .setMaxFullFlushMergeWaitMillis(0));
     TestUtil.reduceOpenFiles(mainWriter);
 
     AddDirectoriesThreads addDirThreads = new AddDirectoriesThreads(numIter, mainWriter);
@@ -422,6 +428,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
       this.mainWriter = mainWriter;
       addDir = newDirectory();
       IndexWriter writer = new IndexWriter(addDir, newIndexWriterConfig(new MockAnalyzer(random()))
+                                                     .setMaxFullFlushMergeWaitMillis(0)
                                                      .setMaxBufferedDocs(2));
       TestUtil.reduceOpenFiles(writer);
       for (int i = 0; i < NUM_INIT_DOCS; i++) {
@@ -534,7 +541,8 @@ public class TestIndexWriterReader extends LuceneTestCase {
    */
   public void doTestIndexWriterReopenSegment(boolean doFullMerge) throws Exception {
     Directory dir1 = getAssertNoDeletesDirectory(newDirectory());
-    IndexWriter writer = new IndexWriter(dir1, newIndexWriterConfig(new MockAnalyzer(random())));
+    IndexWriter writer = new IndexWriter(dir1, newIndexWriterConfig(new MockAnalyzer(random()))
+        .setMaxFullFlushMergeWaitMillis(0));
     IndexReader r1 = writer.getReader();
     assertEquals(0, r1.maxDoc());
     createIndexNoClose(false, "index1", writer);
@@ -570,7 +578,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
     writer.close();
 
     // test whether the changes made it to the directory
-    writer = new IndexWriter(dir1, newIndexWriterConfig(new MockAnalyzer(random())));
+    writer = new IndexWriter(dir1, newIndexWriterConfig(new MockAnalyzer(random())).setMaxFullFlushMergeWaitMillis(0));
     IndexReader w2r1 = writer.getReader();
     // insure the deletes were actually flushed to the directory
     assertEquals(200, w2r1.maxDoc());
@@ -620,6 +628,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
         dir1,
         newIndexWriterConfig(new MockAnalyzer(random()))
           .setMaxBufferedDocs(2)
+          .setMaxFullFlushMergeWaitMillis(0)
           .setMergedSegmentWarmer((leafReader) -> warmCount.incrementAndGet())
           .setMergeScheduler(new ConcurrentMergeScheduler())
           .setMergePolicy(newLogMergePolicy())
@@ -654,7 +663,8 @@ public class TestIndexWriterReader extends LuceneTestCase {
   public void testAfterCommit() throws Exception {
     Directory dir1 = getAssertNoDeletesDirectory(newDirectory());
     IndexWriter writer = new IndexWriter(dir1, newIndexWriterConfig(new MockAnalyzer(random()))
-                                                 .setMergeScheduler(new ConcurrentMergeScheduler()));
+                                                 .setMergeScheduler(new ConcurrentMergeScheduler())
+                                                 .setMaxFullFlushMergeWaitMillis(0));
     writer.commit();
 
     // create the index
@@ -686,7 +696,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
   // Make sure reader remains usable even if IndexWriter closes
   public void testAfterClose() throws Exception {
     Directory dir1 = getAssertNoDeletesDirectory(newDirectory());
-    IndexWriter writer = new IndexWriter(dir1, newIndexWriterConfig(new MockAnalyzer(random())));
+    IndexWriter writer = new IndexWriter(dir1, newIndexWriterConfig(new MockAnalyzer(random())).setMaxFullFlushMergeWaitMillis(0));
 
     // create the index
     createIndexNoClose(false, "test", writer);
@@ -700,7 +710,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
     assertEquals(100, r.numDocs());
     Query q = new TermQuery(new Term("indexname", "test"));
     IndexSearcher searcher = newSearcher(r);
-    assertEquals(100, searcher.search(q, 10).totalHits);
+    assertEquals(100, searcher.count(q));
 
     expectThrows(AlreadyClosedException.class, () -> {
       DirectoryReader.openIfChanged(r);
@@ -716,7 +726,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
     Directory dir1 = getAssertNoDeletesDirectory(newDirectory());
     final IndexWriter writer = new IndexWriter(
         dir1,
-        newIndexWriterConfig(new MockAnalyzer(random()))
+        newIndexWriterConfig(new MockAnalyzer(random())).setMaxFullFlushMergeWaitMillis(0)
           .setMergePolicy(newLogMergePolicy(2))
     );
 
@@ -768,7 +778,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
         r = r2;
         Query q = new TermQuery(new Term("indexname", "test"));
         IndexSearcher searcher = newSearcher(r);
-        final long count = searcher.search(q, 10).totalHits;
+        final long count = searcher.count(q);
         assertTrue(count >= lastCount);
         lastCount = count;
       }
@@ -785,7 +795,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
     }
     Query q = new TermQuery(new Term("indexname", "test"));
     IndexSearcher searcher = newSearcher(r);
-    final long count = searcher.search(q, 10).totalHits;
+    final long count = searcher.count(q);
     assertTrue(count >= lastCount);
 
     assertEquals(0, excs.size());
@@ -810,11 +820,16 @@ public class TestIndexWriterReader extends LuceneTestCase {
   // Stress test reopen during add/delete
   public void testDuringAddDelete() throws Exception {
     Directory dir1 = newDirectory();
+    IndexWriterConfig iwc = newIndexWriterConfig(new MockAnalyzer(random()))
+        .setMergePolicy(newLogMergePolicy(2));
+    if (TEST_NIGHTLY) {
+      // if we have a ton of iterations we need to make sure we don't do unnecessary
+      // extra flushing otherwise we will timeout on nightly
+      iwc.setRAMBufferSizeMB(IndexWriterConfig.DEFAULT_RAM_BUFFER_SIZE_MB);
+      iwc.setMaxBufferedDocs(IndexWriterConfig.DISABLE_AUTO_FLUSH);
+    }
     final IndexWriter writer = new IndexWriter(
-        dir1,
-        newIndexWriterConfig(new MockAnalyzer(random()))
-          .setMergePolicy(newLogMergePolicy(2))
-    );
+        dir1,iwc);
 
     // create the index
     createIndexNoClose(false, "test", writer);
@@ -823,7 +838,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
     DirectoryReader r = writer.getReader();
 
     final int iters = TEST_NIGHTLY ? 1000 : 10;
-    final List<Throwable> excs = Collections.synchronizedList(new ArrayList<Throwable>());
+    final List<Throwable> excs = Collections.synchronizedList(new ArrayList<>());
 
     final Thread[] threads = new Thread[numThreads];
     final AtomicInteger remainingThreads = new AtomicInteger(numThreads);
@@ -865,7 +880,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
         r = r2;
         Query q = new TermQuery(new Term("indexname", "test"));
         IndexSearcher searcher = newSearcher(r);
-        sum += searcher.search(q, 10).totalHits;
+        sum += searcher.count(q);
       }
     }
 
@@ -880,7 +895,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
     }
     Query q = new TermQuery(new Term("indexname", "test"));
     IndexSearcher searcher = newSearcher(r);
-    sum += searcher.search(q, 10).totalHits;
+    sum += searcher.count(q);
     assertTrue("no documents found at all", sum > 0);
 
     assertEquals(0, excs.size());
@@ -965,8 +980,8 @@ public class TestIndexWriterReader extends LuceneTestCase {
            .setReaderPooling(true)
            .setMergedSegmentWarmer((r) -> {
               IndexSearcher s = newSearcher(r);
-              TopDocs hits = s.search(new TermQuery(new Term("foo", "bar")), 10);
-              assertEquals(20, hits.totalHits);
+              int count = s.count(new TermQuery(new Term("foo", "bar")));
+              assertEquals(20, count);
               didWarm.set(true);
            })
            .setMergePolicy(newLogMergePolicy(10))
@@ -1027,7 +1042,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
     Directory d = getAssertNoDeletesDirectory(newDirectory());
     IndexWriter w = new IndexWriter(
         d,
-        newIndexWriterConfig(new MockAnalyzer(random())));
+        newIndexWriterConfig(new MockAnalyzer(random())).setMaxFullFlushMergeWaitMillis(0));
 
     DirectoryReader r = w.getReader(); // start pooling readers
 
@@ -1068,23 +1083,20 @@ public class TestIndexWriterReader extends LuceneTestCase {
     dir.failOn(new MockDirectoryWrapper.Failure() {
       @Override
       public void eval(MockDirectoryWrapper dir) throws IOException {
-        StackTraceElement[] trace = new Exception().getStackTrace();
         if (shouldFail.get()) {
-          for (int i = 0; i < trace.length; i++) {
-            if ("getReadOnlyClone".equals(trace[i].getMethodName())) {
-              if (VERBOSE) {
-                System.out.println("TEST: now fail; exc:");
-                new Throwable().printStackTrace(System.out);
-              }
-              shouldFail.set(false);
-              throw new FakeIOException();
+          if (callStackContainsAnyOf("getReadOnlyClone")) {
+            if (VERBOSE) {
+              System.out.println("TEST: now fail; exc:");
+              new Throwable().printStackTrace(System.out);
             }
+            shouldFail.set(false);
+            throw new FakeIOException();
           }
         }
       }
     });
     
-    IndexWriterConfig conf = newIndexWriterConfig(new MockAnalyzer(random()));
+    IndexWriterConfig conf = newIndexWriterConfig(new MockAnalyzer(random())).setMaxFullFlushMergeWaitMillis(0);
     conf.setMergePolicy(NoMergePolicy.INSTANCE); // prevent merges from getting in the way
     IndexWriter writer = new IndexWriter(dir, conf);
     
@@ -1115,7 +1127,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
     Directory dir = getAssertNoDeletesDirectory(new RAMDirectory());
     // Don't use newIndexWriterConfig, because we need a
     // "sane" mergePolicy:
-    IndexWriterConfig iwc = new IndexWriterConfig(new MockAnalyzer(random()));
+    IndexWriterConfig iwc = new IndexWriterConfig(new MockAnalyzer(random())).setMaxFullFlushMergeWaitMillis(0);
     IndexWriter w = new IndexWriter(dir, iwc);
     // Create 500 segments:
     for(int i=0;i<500;i++) {
@@ -1156,4 +1168,149 @@ public class TestIndexWriterReader extends LuceneTestCase {
     w.close();
     dir.close();
   }
+
+  public void testIndexReaderWriterWithLeafSorter() throws IOException {
+    final String FIELD_NAME = "field1";
+    final boolean ASC_SORT = randomBoolean();
+    final long MISSING_VALUE =
+        ASC_SORT ? Long.MAX_VALUE : Long.MIN_VALUE; // missing values at the end
+
+    // create a comparator that sort leaf readers according with
+    // the min value (asc sort) or max value (desc sort) of its points
+    Comparator<LeafReader> leafSorter = Comparator.comparingLong(
+        r -> {
+          try {
+            PointValues points = r.getPointValues(FIELD_NAME);
+            if (points != null) {
+              byte[] sortValue =
+                  ASC_SORT ? points.getMinPackedValue() : points.getMaxPackedValue();
+              return LongPoint.decodeDimension(sortValue, 0);
+            }
+          } catch (IOException e) {
+          }
+          return MISSING_VALUE;
+        });
+    if (ASC_SORT == false) {
+      leafSorter = leafSorter.reversed();
+    }
+
+    final int NUM_DOCS = atLeast(30);
+    Directory dir = newDirectory();
+    IndexWriterConfig iwc = new IndexWriterConfig();
+    iwc.setLeafSorter(leafSorter);
+    IndexWriter writer = new IndexWriter(dir, iwc);
+    for (int i = 0; i < NUM_DOCS; ++i) {
+      final Document doc = new Document();
+      doc.add(new LongPoint(FIELD_NAME, randomLongBetween(1, 99)));
+      writer.addDocument(doc);
+      if (i > 0 && i % 10 == 0) writer.flush();
+    }
+
+    // Test1: test that leafReaders are sorted according to leafSorter provided in IndexWriterConfig
+    {
+      try (DirectoryReader reader = writer.getReader()) {
+        List<LeafReader> lrs =
+            reader.leaves().stream().map(LeafReaderContext::reader).collect(toList());
+        List<LeafReader> expectedSortedlrs =
+            reader.leaves().stream()
+                .map(LeafReaderContext::reader)
+                .sorted(leafSorter)
+                .collect(toList());
+        assertEquals(expectedSortedlrs, lrs);
+
+        // add more documents that should be sorted first
+        final long FIRST_VALUE = ASC_SORT ? 0 : 100;
+        for (int i = 0; i < 10; ++i) {
+          final Document doc = new Document();
+          doc.add(new LongPoint(FIELD_NAME, FIRST_VALUE));
+          writer.addDocument(doc);
+        }
+        writer.commit();
+
+        // and open again
+        try (DirectoryReader reader2 = DirectoryReader.openIfChanged(reader)) {
+          lrs = reader2.leaves().stream().map(LeafReaderContext::reader).collect(toList());
+          expectedSortedlrs =
+              reader2.leaves().stream()
+                  .map(LeafReaderContext::reader)
+                  .sorted(leafSorter)
+                  .collect(toList());
+          assertEquals(expectedSortedlrs, lrs);
+        }
+      }
+    }
+
+    // Test2: test that leafReaders are sorted according to leafSorter provided in DirectoryReader
+    {
+      try (DirectoryReader reader = DirectoryReader.open(dir, leafSorter)) {
+        List<LeafReader> lrs =
+            reader.leaves().stream().map(LeafReaderContext::reader).collect(toList());
+        List<LeafReader> expectedSortedlrs =
+            reader.leaves().stream()
+                .map(LeafReaderContext::reader)
+                .sorted(leafSorter)
+                .collect(toList());
+        assertEquals(expectedSortedlrs, lrs);
+
+        // add more documents that should be sorted first
+        final long FIRST_VALUE = ASC_SORT ? 0 : 100;
+        for (int i = 0; i < 10; ++i) {
+          final Document doc = new Document();
+          doc.add(new LongPoint(FIELD_NAME, FIRST_VALUE));
+          writer.addDocument(doc);
+        }
+        writer.commit();
+
+        // and open again
+        try (DirectoryReader reader2 = DirectoryReader.openIfChanged(reader)) {
+          lrs = reader2.leaves().stream().map(LeafReaderContext::reader).collect(toList());
+          expectedSortedlrs =
+              reader2.leaves().stream()
+                  .map(LeafReaderContext::reader)
+                  .sorted(leafSorter)
+                  .collect(toList());
+          assertEquals(expectedSortedlrs, lrs);
+        }
+      }
+    }
+
+    // Test3: test that FilterDirectoryReader sorts leaves according
+    // to leafSorter of its wrapped reader
+    {
+      try (DirectoryReader reader =
+               new AssertingDirectoryReader(DirectoryReader.open(dir, leafSorter))) {
+        List<LeafReader> lrs =
+            reader.leaves().stream().map(LeafReaderContext::reader).collect(toList());
+        List<LeafReader> expectedSortedlrs =
+            reader.leaves().stream()
+                .map(LeafReaderContext::reader)
+                .sorted(leafSorter)
+                .collect(toList());
+        assertEquals(expectedSortedlrs, lrs);
+
+        // add more documents that should be sorted first
+        final long FIRST_VALUE = ASC_SORT ? 0 : 100;
+        for (int i = 0; i < 10; ++i) {
+          final Document doc = new Document();
+          doc.add(new LongPoint(FIELD_NAME, FIRST_VALUE));
+          writer.addDocument(doc);
+        }
+        writer.commit();
+
+        // and open again
+        try (DirectoryReader reader2 = DirectoryReader.openIfChanged(reader)) {
+          lrs = reader2.leaves().stream().map(LeafReaderContext::reader).collect(toList());
+          expectedSortedlrs =
+              reader2.leaves().stream()
+                  .map(LeafReaderContext::reader)
+                  .sorted(leafSorter)
+                  .collect(toList());
+          assertEquals(expectedSortedlrs, lrs);
+        }
+      }
+    }
+
+    writer.close();
+    dir.close();
+  }
 }
